<StackLayout class="p-10">


    <GridLayout class="peek-nav-section"
                rows="auto" columns="auto, *, auto">
        <!--
        -->
    </GridLayout>
    <StackLayout class="hr"></StackLayout>

    <StackLayout class="peek-information-section">

        <button class="btn"
                [class.bg-success]="gridLoaderStatus.cacheForOfflineEnabled"
                [text]="gridLoaderStatus.cacheForOfflineEnabled ? 'Disable Offline Support':'Enable Offline Support'"
                (tap)="toggleOfflineMode()"></button>

        <label text="Grid Cache Offline Mode Enabled : {{gridLoaderStatus.cacheForOfflineEnabled}}" textWrap="true"></label>
        <label text="Grid Cache Load Complete : {{gridLoaderStatus.initialLoadComplete}}" textWrap="true"></label>
        <label text="Grid Cache Load Progress : {{gridLoaderStatus.loadProgress}}/{{gridLoaderStatus.loadTotal}}" textWrap="true"></label>
        <label text="Grid Cache Last Check Finished : {{gridLoaderStatus.lastCheck}}" textWrap="true"></label>

        <label text="Location Object Offline Mode Enabled : {{locationLoaderStatus.cacheForOfflineEnabled}}" textWrap="true"></label>
        <label text="Location Object Load Complete : {{locationLoaderStatus.initialLoadComplete}}" textWrap="true"></label>
        <label text="Location Object Load Progress : {{locationLoaderStatus.loadProgress}}/{{locationLoaderStatus.loadTotal}}" textWrap="true"></label>
        <label text="Location Object Last Check : {{locationLoaderStatus.lastCheck}}" textWrap="true"></label>

    </StackLayout>
</StackLayout>