import {Component, OnInit} from "@angular/core";

@Component({
    selector: 'diagram-admin',
    templateUrl: 'diagram.component.html'
})
export class DiagramComponent  implements OnInit {

    ngOnInit() {

    }
}
