/* On the desktop, this slides in from the right.
* https://www.w3schools.com/howto/howto_js_sidenav.asp
*/
@import '~@synerty/peek-theme/scss/_mixins.web';
@import '~@synerty/peek-theme/scss/_variables.web';


$panel_width: 300px;


/* The side navigation menu */
.panel {
  height: 100%; /* 100% Full-height */
  position: fixed; /* Stay in place */
  z-index: 4; /* Stay on top */
  //top: 0;
  right: 0;
  overflow-x: hidden; /* Disable horizontal scroll */
  transition: 0.5s; /* 0.5 second transition effect to slide in the toolbar */
}

/* Set the style for when the toolbar is open */
.is-open {
  min-width: $panel_width;
  max-width: 40vw;
}

/* Set the style for when the toolbar is closed */
.is-closed {
  width: 0;
  border: 0;
}


.panel-header {
  padding: 15px;
}

.close {
  //padding: 6px;
}