@import "../edit-props-component/edit-props.component.web";

.canvas-status-bar {
  z-index: 2;
  position: absolute;
  margin: 0 auto;
  color: grey;
  right: 10px;
}

canvas {
  width: 100%;
  z-index: 1;
}
